@@ -1,7 +1,7 @@
 ﻿/*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2014-2015 Vaclav Slavik
+ *  Copyright (C) 2014-2016 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -28,8 +28,11 @@
 #include "catalog.h"
 #include "customcontrols.h"
 #include "commentdlg.h"
+#include "concurrency.h"
 #include "errors.h"
 #include "hidpi.h"
+#include "utility.h"
+#include "unicode_helpers.h"
 
 #include "tm/suggestions.h"
 #include "tm/transmem.h"
@@ -42,6 +45,7 @@
 #include <wx/sizer.h>
 #include <wx/statbmp.h>
 #include <wx/stattext.h>
+#include <wx/time.h>
 #include <wx/wupdlock.h>
 
 #include <algorithm>
@@ -67,6 +71,8 @@ public:
         return wxSize(-1, 1);
     }
 
+    bool AcceptsFocus() const override { return false; }
+
 private:
     void OnPaint(wxPaintEvent&)
     {
@@ -94,7 +100,7 @@ SidebarBlock::SidebarBlock(Sidebar *parent, const wxString& label, int flags)
                          wxSizerFlags().Expand().Border(wxBOTTOM|wxLEFT, PX(2)));
         }
         m_headerSizer = new wxBoxSizer(wxHORIZONTAL);
-        m_headerSizer->Add(new HeadingLabel(parent, label), wxSizerFlags().Expand().Center());
+        m_headerSizer->Add(new HeadingLabel(parent, label), wxSizerFlags().Expand());
         m_sizer->Add(m_headerSizer, wxSizerFlags().Expand().PXDoubleBorder(wxLEFT|wxRIGHT));
     }
     m_innerSizer = new wxBoxSizer(wxVERTICAL);
@@ -106,7 +112,7 @@ void SidebarBlock::Show(bool show)
     m_sizer->ShowItems(show);
 }
 
-void SidebarBlock::SetItem(CatalogItem *item)
+void SidebarBlock::SetItem(const CatalogItemPtr& item)
 {
     if (!item)
     {
@@ -136,15 +142,14 @@ public:
         m_innerSizer->Add(m_text, wxSizerFlags().Expand());
     }
 
-    bool ShouldShowForItem(CatalogItem *item) const override
+    bool ShouldShowForItem(const CatalogItemPtr& item) const override
     {
         return item->HasOldMsgid();
     }
 
-    void Update(CatalogItem *item) override
+    void Update(const CatalogItemPtr& item) override
     {
-        auto txt = wxJoin(item->GetOldMsgid(), ' ', '\0');
-        m_text->SetAndWrapLabel(txt);
+        m_text->SetAndWrapLabel(item->GetOldMsgid());
     }
 
 private:
@@ -163,12 +168,12 @@ public:
         m_innerSizer->Add(m_comment, wxSizerFlags().Expand());
     }
 
-    bool ShouldShowForItem(CatalogItem *item) const override
+    bool ShouldShowForItem(const CatalogItemPtr& item) const override
     {
         return item->HasExtractedComments();
     }
 
-    void Update(CatalogItem *item) override
+    void Update(const CatalogItemPtr& item) override
     {
         auto comment = wxJoin(item->GetExtractedComments(), '\n', '\0');
         if (comment.StartsWith("TRANSLATORS:") || comment.StartsWith("translators:"))
@@ -196,12 +201,12 @@ public:
         m_innerSizer->Add(m_comment, wxSizerFlags().Expand());
     }
 
-    bool ShouldShowForItem(CatalogItem *item) const override
+    bool ShouldShowForItem(const CatalogItemPtr& item) const override
     {
         return item->HasComment();
     }
 
-    void Update(CatalogItem *item) override
+    void Update(const CatalogItemPtr& item) override
     {
         auto text = CommentDialog::RemoveStartHash(item->GetComment());
         text.Trim();
@@ -230,9 +235,12 @@ public:
 
     bool IsGrowable() const override { return true; }
 
-    bool ShouldShowForItem(CatalogItem*) const override { return true; }
+    bool ShouldShowForItem(const CatalogItemPtr&) const override
+    {
+        return m_parent->FileHasCapability(Catalog::Cap::UserComments);
+    }
 
-    void Update(CatalogItem *item) override
+    void Update(const CatalogItemPtr& item) override
     {
     #ifdef __WXMSW__
         auto add = _("Add comment");
@@ -251,13 +259,11 @@ private:
 
 wxDEFINE_EVENT(EVT_SUGGESTION_SELECTED, wxCommandEvent);
 
-class SuggestionWidget : public wxPanel
+class SuggestionWidget : public wxWindow
 {
 public:
-    SuggestionWidget(wxWindow *parent) : wxPanel(parent, wxID_ANY)
+    SuggestionWidget(wxWindow *parent) : wxWindow(parent, wxID_ANY)
     {
-        m_appIsRTL = (wxTheApp->GetLayoutDirection() == wxLayout_RightToLeft);
-
         m_icon = new wxStaticBitmap(this, wxID_ANY, wxNullBitmap);
         m_text = new AutoWrappingText(this, "TEXT");
         m_info = new InfoStaticText(this);
@@ -284,7 +290,7 @@ public:
         }
     }
 
-    void SetValue(int index, const Suggestion& s, bool isRTL, const wxBitmap& icon, const wxString& tooltip)
+    void SetValue(int index, const Suggestion& s, Language lang, const wxBitmap& icon, const wxString& tooltip)
     {
         m_value = s;
 
@@ -308,19 +314,9 @@ public:
 
         m_icon->SetBitmap(icon);
 
-        auto text = wxControl::EscapeMnemonics(s.text);
-        if (isRTL)
-            text = L"\u202b" + text;
-        else
-            text = L"\u202a" + text;
-        m_text->SetAndWrapLabel(text);
+        auto text = wxControl::EscapeMnemonics(bidi::mark_direction(s.text, lang));
 
-        // a quirk of wx API: if the current locale is RTL, the meaning of L and R is reversed
-        // for alignments
-        if (m_appIsRTL)
-            isRTL = !isRTL;
-
-        m_text->SetAlignment(isRTL ? wxALIGN_RIGHT : wxALIGN_LEFT);
+        m_text->SetLanguage(lang);
         m_text->SetAndWrapLabel(text);
 
 #ifndef __WXOSX__
@@ -336,6 +332,8 @@ public:
         SetMinSize(wxDefaultSize);
         SetMinSize(GetBestSize());
     }
+    
+    bool AcceptsFocus() const override { return false; }
 
 private:
     class InfoStaticText : public wxStaticText
@@ -380,7 +378,6 @@ private:
     AutoWrappingText *m_text;
     wxStaticText *m_info;
     wxColour m_bg, m_bgHighlight;
-    bool m_appIsRTL;
 };
 
 
@@ -389,7 +386,8 @@ SuggestionsSidebarBlock::SuggestionsSidebarBlock(Sidebar *parent, wxMenu *menu)
       m_suggestionsMenu(menu),
       m_msgPresent(false),
       m_pendingQueries(0),
-      m_latestQueryId(0)
+      m_latestQueryId(0),
+      m_lastUpdateTime(0)
 {
     m_provider.reset(new SuggestionsProvider);
 
@@ -424,10 +422,18 @@ SuggestionsSidebarBlock::SuggestionsSidebarBlock(Sidebar *parent, wxMenu *menu)
     m_innerSizer->Add(m_iGotNothing, wxSizerFlags().Center().Border(wxTOP|wxBOTTOM, PX(100)));
 
     BuildSuggestionsMenu();
+
+    m_suggestionsTimer.SetOwner(parent);
+    parent->Bind(wxEVT_TIMER,
+                 &SuggestionsSidebarBlock::OnDelayedShowSuggestionsForItem, this,
+                 m_suggestionsTimer.GetId());
 }
 
 SuggestionsSidebarBlock::~SuggestionsSidebarBlock()
 {
+    ClearSuggestionsMenu();
+    for (auto i : m_suggestionMenuItems)
+        delete i;
 }
 
 wxBitmap SuggestionsSidebarBlock::GetIconForSuggestion(const Suggestion&) const
@@ -464,7 +470,6 @@ void SuggestionsSidebarBlock::ReportError(SuggestionsBackend*, std::exception_pt
 
 void SuggestionsSidebarBlock::ClearSuggestions()
 {
-    m_pendingQueries = 0;
     m_suggestions.clear();
     UpdateSuggestionsMenu();
     UpdateVisibility();
@@ -493,11 +498,11 @@ void SuggestionsSidebarBlock::UpdateSuggestions(const SuggestionsList& hits)
     m_innerSizer->Layout();
 
     // update shown suggestions:
-    bool isRTL = m_parent->GetCurrentLanguage().IsRTL();
+    auto lang = m_parent->GetCurrentLanguage();
     for (size_t i = 0; i < m_suggestions.size(); ++i)
     {
         auto s = m_suggestions[i];
-        m_suggestionsWidgets[i]->SetValue((int)i, s, isRTL, GetIconForSuggestion(s), GetTooltipForSuggestion(s));
+        m_suggestionsWidgets[i]->SetValue((int)i, s, lang, GetIconForSuggestion(s), GetTooltipForSuggestion(s));
     }
 
     m_innerSizer->Layout();
@@ -548,12 +553,17 @@ void SuggestionsSidebarBlock::UpdateSuggestionsMenu()
         if (index >= SUGGESTIONS_MENU_ENTRIES)
             break;
 
-        auto text = wxString::Format(formatMask, s.text, index+1);
+        wxString text = s.text;
+        text.Replace("\t", " ");
+        text.Replace("\n", " ");
+        if (text.length() > 100)
+            text = text.substr(0, 100) + L"…";
 
         auto item = m_suggestionMenuItems[index];
         m_suggestionsMenu->Append(item);
 
-        item->SetItemLabel(text);
+        auto label = wxControl::EscapeMnemonics(wxString::Format(formatMask, text, index+1));
+        item->SetItemLabel(label);
         item->SetBitmap(GetIconForSuggestion(s));
 
         index++;
@@ -615,46 +625,84 @@ void SuggestionsSidebarBlock::Show(bool show)
     }
 }
 
-bool SuggestionsSidebarBlock::ShouldShowForItem(CatalogItem*) const
+bool SuggestionsSidebarBlock::ShouldShowForItem(const CatalogItemPtr&) const
 {
-    return wxConfig::Get()->ReadBool("use_tm", true);
+    return m_parent->FileHasCapability(Catalog::Cap::Translations) &&
+           wxConfig::Get()->ReadBool("use_tm", true);
 }
 
-void SuggestionsSidebarBlock::Update(CatalogItem *item)
+void SuggestionsSidebarBlock::Update(const CatalogItemPtr& item)
 {
-    // FIXME: Cancel previous pending async operation if any
-
     ClearMessage();
     ClearSuggestions();
 
+    UpdateSuggestionsForItem(item);
+}
+
+void SuggestionsSidebarBlock::UpdateSuggestionsForItem(CatalogItemPtr item)
+{
+    if (!item)
+        return;
+
+    long long now = wxGetUTCTimeMillis().GetValue();
+    long long delta = now - m_lastUpdateTime;
+    m_lastUpdateTime = now;
+
+    if (delta < 100)
+    {
+        // User is probably holding arrow down and going through the list as crazy
+        // and not really caring for the suggestions. Throttle them a bit and call
+        // this code after a small delay. Notice that this may repeat itself several
+        // times, only continuing through to show suggestions after the dust settled
+        // and the user didn't change the selection for a few milliseconds.
+        if (!m_suggestionsTimer.IsRunning())
+            m_suggestionsTimer.StartOnce(110);
+        return;
+    }
+
+    m_pendingQueries = 0;
+
+    auto srclang = m_parent->GetCurrentSourceLanguage();
+    auto lang = m_parent->GetCurrentLanguage();
+    if (!srclang.IsValid() || !lang.IsValid() || srclang == lang)
+    {
+        OnQueriesFinished();
+        return;
+    }
+
     QueryAllProviders(item);
 }
 
-void SuggestionsSidebarBlock::QueryAllProviders(CatalogItem *item)
+void SuggestionsSidebarBlock::OnDelayedShowSuggestionsForItem(wxTimerEvent&)
+{
+    UpdateSuggestionsForItem(m_parent->GetSelectedItem());
+}
+
+void SuggestionsSidebarBlock::QueryAllProviders(const CatalogItemPtr& item)
 {
     auto thisQueryId = ++m_latestQueryId;
     QueryProvider(TranslationMemory::Get(), item, thisQueryId);
 }
 
-void SuggestionsSidebarBlock::QueryProvider(SuggestionsBackend& backend, CatalogItem *item, uint64_t queryId)
+void SuggestionsSidebarBlock::QueryProvider(SuggestionsBackend& backend, const CatalogItemPtr& item, uint64_t queryId)
 {
     m_pendingQueries++;
 
     // we need something to talk to GUI thread through that is guaranteed
     // to exist, and the app object is a good choice:
-    wxApp *app = wxTheApp;
     auto backendPtr = &backend;
     std::weak_ptr<SuggestionsSidebarBlock> weakSelf = std::dynamic_pointer_cast<SuggestionsSidebarBlock>(shared_from_this());
 
     m_provider->SuggestTranslation
     (
         backend,
+        m_parent->GetCurrentSourceLanguage(),
         m_parent->GetCurrentLanguage(),
         item->GetString().ToStdWstring(),
 
         // when receiving data
         [=](const SuggestionsList& hits){
-            app->CallAfter([weakSelf,queryId,hits]{
+            call_on_main_thread([weakSelf,queryId,hits]{
                 auto self = weakSelf.lock();
                 // maybe this call is already out of date:
                 if (!self || self->m_latestQueryId != queryId)
@@ -667,7 +715,7 @@ void SuggestionsSidebarBlock::QueryProvider(SuggestionsBackend& backend, Catalog
 
         // on error:
         [=](std::exception_ptr e){
-            app->CallAfter([weakSelf,queryId,backendPtr,e]{
+            call_on_main_thread([weakSelf,queryId,backendPtr,e]{
                 auto self = weakSelf.lock();
                 // maybe this call is already out of date:
                 if (!self || self->m_latestQueryId != queryId)
@@ -683,11 +731,16 @@ void SuggestionsSidebarBlock::QueryProvider(SuggestionsBackend& backend, Catalog
 
 
 Sidebar::Sidebar(wxWindow *parent, wxMenu *suggestionsMenu)
-    : wxPanel(parent, wxID_ANY),
+    : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxNO_BORDER | DoubleBufferingWindowStyle()),
       m_catalog(nullptr),
       m_selectedItem(nullptr)
 {
     SetBackgroundColour(SIDEBAR_BACKGROUND);
+#ifdef __WXMSW__
+    if (!IsWindowsXP())
+        SetDoubleBuffered(true);
+#endif
+
     Bind(wxEVT_PAINT, &Sidebar::OnPaint, this);
 #ifdef __WXOSX__
     SetWindowVariant(wxWINDOW_VARIANT_SMALL);
@@ -732,7 +785,7 @@ Sidebar::~Sidebar()
 }
 
 
-void Sidebar::SetSelectedItem(Catalog *catalog, CatalogItem *item)
+void Sidebar::SetSelectedItem(const CatalogPtr& catalog, const CatalogItemPtr& item)
 {
     m_catalog = catalog;
     m_selectedItem = item;
@@ -751,6 +804,18 @@ Language Sidebar::GetCurrentLanguage() const
     return m_catalog->GetLanguage();
 }
 
+Language Sidebar::GetCurrentSourceLanguage() const
+{
+    if (!m_catalog)
+        return Language::English();
+    return m_catalog->GetSourceLanguage();
+}
+
+bool Sidebar::FileHasCapability(Catalog::Cap cap) const
+{
+    return m_catalog && m_catalog->HasCapability(cap);
+}
+
 void Sidebar::RefreshContent()
 {
     if (!IsShown())
