@@ -830,7 +830,7 @@ void TranslationTextCtrl::SetPlainTextUserWritten(const wxString& value)
     DisableAutomaticSubstitutions disableAuto(this);
 #endif
 
-    Clear();
+    SelectAll();
     WriteText(EscapePlainText(value));
     SetInsertionPointEnd();
 }
